<?php
/**
 * Plugin main class file
 *
 * @package StandaloneTech
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}
/**
 * Plugin main class
 */
final class WOO_WALLET_IMPORTER {

	/**
	 * The single instance of the class.
	 *
	 * @var WOO_WALLET_IMPORT
	 * @since 1.0.0
	 */
	protected static $_instance = null;

	/**
	 * Plugin license API class.
	 *
	 * @var Woo_Wallet_License
	 */
	public $licence = null;

	/**
	 * Main instance
	 *
	 * @return class object
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Class constructor
	 */
	public function __construct() {
		if ( Woo_Wallet_Dependencies::is_woo_wallet_active() ) {
			$this->init_hooks();
			add_action( 'init', array( $this, 'load_plugin_textdomain' ) );
			do_action( 'woo_wallet_importer_loaded' );
		}
	}

	/**
	 * Check request
	 *
	 * @param string $type type.
	 * @return bool
	 */
	private function is_request( $type ) {
		switch ( $type ) {
			case 'admin':
				return is_admin();
			case 'ajax':
				return defined( 'DOING_AJAX' );
			case 'cron':
				return defined( 'DOING_CRON' );
			case 'frontend':
				return ( ! is_admin() || defined( 'DOING_AJAX' ) ) && ! defined( 'DOING_CRON' );
		}
	}

	/**
	 * Plugin init
	 */
	private function init_hooks() {
		add_action( 'admin_init', array( $this, 'admin_init' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'admin_scripts' ), 10 );

		if ( $this->is_request( 'ajax' ) ) {
			add_action( 'wp_ajax_woo_wallet_do_ajax_import', array( $this, 'woo_wallet_do_ajax_import_callback' ) );
		}
	}
	/**
	 * Ajax import callback
	 */
	public function woo_wallet_do_ajax_import_callback() {
		check_ajax_referer( 'woo-wallet-import', 'security' );
		if ( ! isset( $_POST['file'] ) ) {
			wp_send_json_error( array( 'message' => __( 'Insufficient privileges to import.', 'woo-wallet-importer' ) ) );
		}

		$params   = array(
			'delimiter' => ! empty( $_POST['delimiter'] ) ? wc_clean( sanitize_text_field( wp_unslash( $_POST['delimiter'] ) ) ) : ',',
			'start_pos' => isset( $_POST['position'] ) ? absint( $_POST['position'] ) : 2,
			'lines'     => apply_filters( 'woo_wallet_import_batch_size', 20 ),
			'parse'     => true,
			'id'        => isset( $_POST['import_id'] ) ? absint( $_POST['import_id'] ) : 0,
			'imported'  => isset( $_POST['imported'] ) ? absint( $_POST['imported'] ) : 0,
			'failed'    => isset( $_POST['failed'] ) ? absint( $_POST['failed'] ) : 0,
		);
		$messages = get_transient( "woo_wallet_import_messages_{$params['id']}" ) ? get_transient( "woo_wallet_import_messages_{$params['id']}" ) : array();
		$file     = wc_clean( sanitize_text_field( wp_unslash( $_POST['file'] ) ) );

		$h                   = new WOO_Wallet_Importer_Helper();
		$handle              = $h->fopen( $file, 'r' );
		$total_records_count = count( file( $file, FILE_SKIP_EMPTY_LINES ) );
		$response            = array(
			'position'   => $params['start_pos'],
			'imported'   => $params['imported'],
			'failed'     => $params['failed'],
			'percentage' => 0,
			'messages'   => $params['messages'],
		);

		if ( $total_records_count > 0 ) {
			$header_columns = $h->get_row( $file, 1 );
			if ( $params['lines'] > $total_records_count - 1 ) {
				$params['lines'] = $total_records_count - 1;
			}
			for ( $i = $params['start_pos']; $i < ( $params['start_pos'] + $params['lines'] ); $i++ ) {
				$row_data = $h->get_row( $file, $i );
				if ( ! $row_data ) {
					continue;
				}
				$email   = array_search( 'email', $header_columns );
				$amount  = array_search( 'amount', $header_columns );
				$details = array_search( 'details', $header_columns );
				$type    = array_search( 'type', $header_columns );
				$user    = apply_filters( 'woo_wallet_importer_user', get_user_by( 'email', $row_data[ $email ] ), $row_data[ $email ] );
				$amount  = apply_filters( 'woo_wallet_importer_amount', floatval( $row_data[ $amount ] ), $user, $row_data[ $type ], $row_data[ $email ] );
				if ( $user && function_exists( 'woo_wallet' ) ) {
					$price = wc_price( $amount );
					if ( 'debit' === $row_data[ $type ] ) {
						$transaction_id = woo_wallet()->wallet->debit( $user->ID, $amount, $row_data[ $details ] );
						if ( $transaction_id ) {
							/* translators: %1$s: email, %2$s: price */
							$messages[] = sprintf( __( '<strong>%1$s</strong> wallet has debited with %2$s', 'woo-wallet-importer' ), $user->user_email, $price );
						} else {
							/* translators: %s: email */
							$messages[] = sprintf( __( 'An error occurred for this transaction (<strong>%s</strong>)', 'woo-wallet-importer' ), $user->user_email );
						}
					} else {
						$transaction_id = woo_wallet()->wallet->credit( $user->ID, $amount, $row_data[ $details ] );
						if ( $transaction_id ) {
							/* translators: %1$s: email, %2$s: price */
							$messages[] = sprintf( __( '<strong>%1$s</strong> wallet has credited with %2$s', 'woo-wallet-importer' ), $user->user_email, $price );
						} else {
							/* translators: %s: email */
							$messages[] = sprintf( esc_html__( 'An error occurred for this transaction (<strong>%s</strong>)', 'woo-wallet-importer' ), $user->user_email );
						}
					}
				} else {
					/* translators: %s: email */
					$messages[] = sprintf( esc_html__( '<strong>%s</strong> user does not exist' ), $row_data[ $email ] );
				}
				if ( $transaction_id ) {
					$response['imported']++;
				} else {
					$response['failed']++;
				}
				$response['position']++;
			}
		}
		if ( $response['position'] >= $total_records_count + 1 ) {
			$response['position']   = 'done';
			$response['percentage'] = 100;
			$response['url']        = admin_url( "admin.php?import=woocommerce-wallet&step=2&import_id={$params['id']}&imported={$response['imported']}&failed={$response['failed']}" );
			$h->fclose( $handle );
			set_transient( "woo_wallet_import_messages_{$params['id']}", $messages, HOUR_IN_SECONDS );
			wp_import_cleanup( $params['id'] );
		} else {
			$h->fclose( $handle );
			set_transient( "woo_wallet_import_messages_{$params['id']}", $messages, HOUR_IN_SECONDS );
			$response['percentage'] = absint( ( ( $response['position'] - 1 ) * 100 ) / ( $total_records_count + 1 ) );
		}
		wp_send_json_success( $response );
	}

	/**
	 * Load Localisation files.
	 *
	 * Note: the first-loaded translation file overrides any following ones if the same translation is present.
	 */
	public function load_plugin_textdomain() {
		$locale = determine_locale();
		$locale = apply_filters( 'plugin_locale', $locale, 'woo-wallet-importer' );

		unload_textdomain( 'woo-wallet-importer', true );
		load_textdomain( 'woo-wallet-importer', WP_LANG_DIR . '/woo-wallet-importer/woo-wallet-importer-' . $locale . '.mo' );
		load_plugin_textdomain( 'woo-wallet-importer', false, plugin_basename( dirname( WOO_WALLET_IMPORTER_PLUGIN_FILE ) ) . '/languages' );
	}

	/**
	 * Plugin url
	 *
	 * @return string path
	 */
	public function plugin_url() {
		return untrailingslashit( plugins_url( '/', WOO_WALLET_IMPORTER_PLUGIN_FILE ) );
	}

	/**
	 * Admin init
	 */
	public function admin_init() {
		/**
		 * Importer object for registering the import callback
		 *
		 * @global Woo_Wallet_Import $woo_wallet_import
		 */
		if ( defined( 'WP_LOAD_IMPORTERS' ) ) {
			$csv_importer = new Woo_Wallet_CSV_Importer();
			register_importer( 'woocommerce-wallet', 'WooCommerce Wallet', __( 'Import <strong>user balance</strong> from a WordPress export file.', 'woo-wallet-importer' ), array( $csv_importer, 'dispatch' ) );
		}
	}
	/**
	 * Enqueue admin scripts.
	 */
	public function admin_scripts() {
		$screen    = get_current_screen();
		$screen_id = $screen ? $screen->id : '';
		// Register scripts.
		wp_register_script( 'woo_wallet_importer_admin', $this->plugin_url() . '/assets/js/admin.js', array( 'jquery' ), WOO_WALLET_IMPORTER_VERSION, true );
		wp_localize_script(
			'woo_wallet_importer_admin',
			'woo_wallet_importer_admin_param',
			array(
				'import_url' => admin_url( 'admin.php?import=woocommerce-wallet' ),
				'title'      => __(
					'Import',
					'woo-wallet-importer'
				),
			)
		);
		if ( in_array( $screen_id, array( 'toplevel_page_woo-wallet' ), true ) ) {
			wp_enqueue_script( 'woo_wallet_importer_admin' );
		}
	}

}
