<?php
/**
 * TeraWallet license checker
 *
 * @package TeraWallet
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}
if ( ! class_exists( 'Woo_Wallet_License' ) ) {
	/**
	 * TeraWallet license checker class
	 */
	class Woo_Wallet_License {

		/**
		 * Software slug.
		 *
		 * @var string
		 */
		public $software_slug;
		/**
		 * Software key used for option names
		 * and many other places
		 *
		 * @var string
		 */
		public $software_key;
		/**
		 * Software name.
		 *
		 * @var string
		 */
		public $software_name;
		/**
		 * Software version.
		 *
		 * @var string
		 */
		public $software_version;
		/**
		 * Software product id.
		 *
		 * @var string
		 */
		public $product_id;
		/**
		 * Software api url.
		 *
		 * @var string
		 */
		public $api_url;
		/**
		 * Software license key.
		 *
		 * @var string
		 */
		public $license_key = '';
		/**
		 * Class constructor.
		 *
		 * @param string $file file.
		 * @param int    $product_id product_id.
		 * @param string $software_version software_version.
		 * @param string $software_slug software_slug.
		 * @param string $plugin_or_theme plugin_or_theme.
		 * @param string $api_url api_url.
		 */
		public function __construct( $file, $product_id, $software_version, $software_slug = '', $plugin_or_theme = 'plugin', $api_url = 'https://standalonetech.com/' ) {
			if ( ! function_exists( 'get_plugin_data' ) ) {
				require_once ABSPATH . 'wp-admin/includes/plugin.php';
			}
			$plugin_data = get_plugin_data( $file );

			$this->software_slug    = $software_slug;
			$this->software_key     = str_replace( '-', '_', $this->software_slug );
			$this->software_version = $software_version;
			$this->software_name    = $plugin_data['Name'];
			$this->product_id       = $product_id;
			$this->api_url          = $api_url;

			$license_options = get_option( "_wallet_settings_extensions_{$this->software_key}_license", array() );
			if ( ! empty( $license_options ) ) {
				$this->license_key = $license_options['licence_key'];
			}

			add_filter( 'woo_wallet_extensions_settings_sections', array( $this, 'woo_wallet_extensions_settings_sections' ) );
			add_filter( 'woo_wallet_extensions_settings_filds', array( $this, 'woo_wallet_extensions_settings_filds' ) );
			add_action( "update_option__wallet_settings_extensions_{$this->software_key}_license", array( $this, 'activate_or_deactivate_extension_license' ), 10, 3 );

			if ( ! get_option( "{$this->software_key}_license_activated" ) ) {
				add_action( 'admin_notices', array( $this, 'license_inactive_notice' ) );
			}

			// Check for plugin update.
			add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'check_for_software_updates' ) );
			add_filter( 'plugins_api', array( $this, 'request_plugin_information' ), 10, 3 );
		}
		/**
		 * Display plugin deactive error message.
		 *
		 * @return void
		 */
		public function license_inactive_notice() {
			if ( ! current_user_can( 'manage_options' ) ) {
				return;
			}
			if ( isset( $_GET['page'] ) && 'woo-wallet-extensions' == $_GET['page'] ) {
				return;
			}
			?>
			<div class="notice notice-error is-dismissible">
				<p><?php printf( '%1$sClick here%2$s to activate %3$s license key to receive updates and support.', '<a href="' . esc_url( admin_url( 'admin.php?page=woo-wallet-extensions&activewwtab=_wallet_settings_extensions_' . $this->software_key . '_license' ) ) . '">', '</a>', esc_html( $this->software_name ) ); ?></p>
			</div>
			<?php
		}

		/**
		 * Add settings sections to extensions page.
		 *
		 * @param array $sections sections.
		 * @return array
		 */
		public function woo_wallet_extensions_settings_sections( $sections ) {
			$sections[] = array(
				'id'    => "_wallet_settings_extensions_{$this->software_key}_license",
				'title' => "{$this->software_name} License",
				'icon'  => 'dashicons-tickets-alt',
			);
			return $sections;
		}

		/**
		 * Define settings fields for extensions page.
		 *
		 * @param array $settings_fields settings_fields.
		 * @return array
		 */
		public function woo_wallet_extensions_settings_filds( $settings_fields ) {
			$settings_fields[ "_wallet_settings_extensions_{$this->software_key}_license" ] = array(
				array(
					'name'    => 'licence_key',
					'label'   => __( 'API License Key', 'woo-wallet-coupons' ),
					'desc'    => __( 'The license key can be located at <a href="https://standalonetech.com/my-account/api-keys/">https://standalonetech.com/my-account/api-keys/</a>', 'woo-wallet-coupons' ),
					'type'    => 'text',
					'default' => '',
				),
				array(
					'name'  => 'is_activate',
					'label' => __( 'Deactivate API License Key', 'woo-wallet-coupons' ),
					'desc'  => __( 'Deactivates an API License Key so it can be used on another blog.', 'woo-wallet-coupons' ),
					'type'  => 'checkbox',
				),
				array(
					'name' => 'nonce_rand',
					'type' => 'rand',
				),
			);
			return $settings_fields;
		}

		/**
		 * Activate or deactivate license.
		 *
		 * @param array  $old_value old_value.
		 * @param array  $value value.
		 * @param string $option option.
		 * @return void
		 */
		public function activate_or_deactivate_extension_license( $old_value, $value, $option ) {
			if ( ! empty( $value ) ) {
				$args = array(
					'product_id'        => $this->product_id,
					'api_key'           => $value['licence_key'],
					'activation_status' => $value['is_activate'] ? $value['is_activate'] : 'off',
				);
				// Activate or deactivate license here.
				$activation_status = get_option( "{$this->software_key}_license_activated" );
				if ( 'off' === $args['activation_status'] && ! $activation_status ) {
					$activate_results = $this->activate_software_license( $args['api_key'] );
					if ( ! $activate_results ) {
						add_settings_error( '', esc_attr( "{$this->software_key}_license_activation_message" ), __( 'Connection failed to the License Key API server. Try again later. ' ), 'error' );
						delete_option( "{$this->software_key}_license_activated" );
					}
					if ( isset( $activate_results['activated'] ) && true === $activate_results['activated'] ) {
						add_settings_error( '', esc_attr( "{$this->software_key}_license_activation_message" ), __( 'Plugin license activated. ' ) . "{$activate_results['message']}.", 'success' );
						update_option( "{$this->software_key}_license_activated", true );
					}

					if ( isset( $activate_results['code'] ) ) {
						add_settings_error( '', esc_attr( "{$this->software_key}_license_activation_message" ), $activate_results['error'], 'error' );
						delete_option( "{$this->software_key}_license_activated" );
					}
				} elseif ( 'on' === $args['activation_status'] && $activation_status ) {
					$deactivation_results = $this->deactivate_software_license( $args['api_key'] );
					if ( ! $deactivation_results ) {
						add_settings_error( '', esc_attr( "{$this->software_key}_license_activation_message" ), __( 'Connection failed to the License Key API server. Try again later. ' ), 'error' );
					}
					if ( isset( $deactivation_results['deactivated'] ) && true === $deactivation_results['deactivated'] ) {
						add_settings_error( '', esc_attr( "{$this->software_key}_license_activation_message" ), __( 'Plugin license deactivated.' ), 'updated' );
						delete_option( "{$this->software_key}_license_activated" );
						delete_option( "{$this->software_key}_license_instance" );
						delete_option( "_wallet_settings_extensions_{$this->software_key}_license" );
					}
				} else {
					add_settings_error( '', esc_attr( "{$this->software_key}_license_activation_message" ), __( 'Plugin license already activated.' ), 'success' );
				}
			}
		}

		/**
		 * Generate the software api url
		 *
		 * @param array $args args.
		 * @return string
		 */
		private function generate_software_api_url( $args = array() ) {
			$api_url = add_query_arg( 'wc-api', 'wc-am-api', $this->api_url );
			$api_url = add_query_arg( $args, $api_url );
			return $api_url;
		}

		/**
		 * Get software instance.
		 *
		 * @return string
		 */
		public function get_software_instance() {
			if ( get_option( "{$this->software_key}_license_instance" ) ) {
				return get_option( "{$this->software_key}_license_instance" );
			}
			$license_instance = $this->generate_password( 12, false );
			update_option( "{$this->software_key}_license_instance", $license_instance );
			return $license_instance;
		}

		/**
		 * Activate software license.
		 *
		 * @param string $api_key api_key.
		 * @return array | bool
		 */
		private function activate_software_license( $api_key = '' ) {
			if ( $api_key ) {
				$this->license_key = $api_key;
			}
			$request = wp_remote_get(
				$this->generate_software_api_url(
					array(
						'wc_am_action' => 'activate',
						'instance'     => $this->get_software_instance(),
						'object'       => $this->get_site_url(),
						'product_id'   => $this->product_id,
						'version'      => $this->software_version,
						'api_key'      => $this->license_key,
					)
				),
				array(
					'timeout'   => 120,
					'sslverify' => false,
				)
			);
			if ( is_wp_error( $request ) ) {
				return false;
			}
			return json_decode( wp_remote_retrieve_body( $request ), true );
		}

		/**
		 * Deactivate software license.
		 *
		 * @param string $api_key api_key.
		 * @return array | bool
		 */
		private function deactivate_software_license( $api_key = '' ) {
			if ( $api_key ) {
				$this->license_key = $api_key;
			}
			$request = wp_remote_get(
				$this->generate_software_api_url(
					array(
						'wc_am_action' => 'deactivate',
						'instance'     => $this->get_software_instance(),
						'product_id'   => $this->product_id,
						'api_key'      => $this->license_key,
					)
				),
				array(
					'timeout'   => 120,
					'sslverify' => false,
				)
			);
			if ( is_wp_error( $request ) ) {
				return false;
			}
			return json_decode( wp_remote_retrieve_body( $request ), true );
		}

		/**
		 * Check for software updates.
		 *
		 * @param Object $transient transient.
		 * @return Object
		 */
		public function check_for_software_updates( $transient ) {

			if ( empty( $transient->checked ) ) {
				return $transient;
			}
			$request = wp_remote_get(
				$this->generate_software_api_url(
					array(
						'wc_am_action' => 'update',
						'instance'     => $this->get_software_instance(),
						'product_id'   => $this->product_id,
						'version'      => $this->software_version,
						'plugin_name'  => "{$this->software_slug}/$this->software_slug.php",
						'slug'         => $this->software_slug,
						'api_key'      => $this->license_key,
					)
				),
				array(
					'timeout'   => 120,
					'sslverify' => false,
				)
			);
			if ( ! is_wp_error( $request ) ) {
				$response = json_decode( wp_remote_retrieve_body( $request ) );
				if ( $response->success ) {
					$new_ver = (string) $response->data->package->new_version;
					if ( version_compare( $new_ver, $this->software_version, '>' ) ) {
						$transient->response[ "{$this->software_slug}/$this->software_slug.php" ] = $response->data->package;
					}
				}
			}
			return $transient;
		}

		/**
		 * Get software information.
		 *
		 * @return Object | bool
		 */
		public function get_software_information() {
			$request = wp_remote_get(
				$this->generate_software_api_url(
					array(
						'wc_am_action' => 'information',
						'instance'     => $this->get_software_instance(),
						'product_id'   => $this->product_id,
						'version'      => $this->software_version,
						'plugin_name'  => "{$this->software_slug}/$this->software_slug.php",
						'api_key'      => $this->license_key,
					)
				),
				array(
					'timeout'   => 120,
					'sslverify' => false,
				)
			);
			if ( ! is_wp_error( $request ) ) {
				$response = json_decode( wp_remote_retrieve_body( $request ), true );
				if ( $response->success ) {
					return $response->data->info;
				}
			}
			return false;
		}

		/**
		 * Generate plugin information
		 *
		 * @param Object $_data data.
		 * @param string $_action action.
		 * @param Object $_args args.
		 * @return Object
		 */
		public function request_plugin_information( $_data, $_action = '', $_args = null ) {
			if ( 'plugin_information' !== $_action ) {
				return $_data;
			}

			if ( ! isset( $_args->slug ) || ( $_args->slug !== $this->software_slug ) ) {
				return $_data;
			}

			$cache_key = 'plugin_api_request_' . substr( md5( serialize( $this->software_slug ) ), 0, 15 );

			$api_request_transient = get_site_transient( $cache_key );

			if ( empty( $api_request_transient ) ) {
				$api_response                    = $this->get_software_information();
				$api_request_transient           = new \stdClass();
				$api_request_transient->name     = $this->software_name;
				$api_request_transient->slug     = $this->software_slug;
				$api_request_transient->author   = '<a href="https://standalonetech.com/">StandaloneTech</a>';
				$api_request_transient->homepage = 'https://standalonetech.com/';
				$api_request_transient->requires = $api_response->requires;
				$api_request_transient->tested   = $api_response->tested;

				$api_request_transient->version      = $api_response->version;
				$api_request_transient->last_updated = $api_response->last_updated;
				$api_request_transient->autoupdate   = true;

				$api_request_transient->sections = unserialize( $api_response->sections );

				// Expires in 1 day.
				set_site_transient( $cache_key, $api_request_transient, DAY_IN_SECONDS );
			}

			$_data = $api_request_transient;

			return $_data;
		}

		/**
		 * Deletes all data if plugin deactivated
		 *
		 * @return void
		 */
		public function uninstall() {
			global $blog_id;
			$deactivation_results = $this->deactivate_software_license();
			if ( is_multisite() ) {
				switch_to_blog( $blog_id );
				delete_option( "{$this->software_key}_license_activated" );
				delete_option( "{$this->software_key}_license_instance" );
				delete_option( "_wallet_settings_extensions_{$this->software_key}_license" );
				restore_current_blog();
			} else {
				delete_option( "{$this->software_key}_license_activated" );
				delete_option( "{$this->software_key}_license_instance" );
				delete_option( "_wallet_settings_extensions_{$this->software_key}_license" );
			}
		}

		/**
		 * Generate a random password.
		 *
		 * @param integer $length length.
		 * @param boolean $special_chars special_chars.
		 * @param boolean $extra_special_chars extra_special_chars.
		 * @return string
		 */
		protected function generate_password( $length = 12, $special_chars = true, $extra_special_chars = false ) {
			$chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
			if ( $special_chars ) {
				$chars .= ' ! @// $%^&*()';
			}
			if ( $extra_special_chars ) {
				$chars .= '-_ []{}<>~`+=,.;:/?|';
			}

			$password = '';
			for ( $i = 0; $i < $length; $i++ ) {
				$password .= substr( $chars, wp_rand( 0, strlen( $chars ) - 1 ), 1 );
			}
			return $password;
		}
		/**
		 * Get site url
		 *
		 * @return string
		 */
		public function get_site_url() {
			$url        = site_url();
			$disallowed = array( 'http://', 'https://' );
			foreach ( $disallowed as $d ) {
				if ( strpos( $url, $d ) === 0 ) {
					return str_replace( $d, '', $url );
				}
			}
			return $url;
		}

	}

}
